package com.showmejava.gridbag;

import javax.swing.*;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/**
 * A GUI panel which edits a single Constraints object.
 * @author Eric M. Burke
 * @version 2.0
 */
public class ConstraintsEditorPanel extends JPanel 
implements ActionListener, FocusListener {
    private Constraints constraints;

    private JTextField gridXFld = new JTextField();
    private JTextField gridYFld = new JTextField();
    private JTextField gridWidthFld = new JTextField();
    private JTextField gridHeightFld = new JTextField();
    private JComboBox fillCombo = new JComboBox(GBCUtil.ALL_FILLS);
    private JComboBox anchorCombo = new JComboBox(GBCUtil.ALL_ANCHORS);
    private JTextField nameFld = new JTextField();
    private JTextField weightXFld = new JTextField();
    private JTextField weightYFld = new JTextField();
    private JTextField insetsLeftFld = new JTextField(3);
    private JTextField insetsRightFld = new JTextField(3);
    private JTextField insetsTopFld = new JTextField(3);
    private JTextField insetsBottomFld = new JTextField(3);
    private JTextField iPadXFld = new JTextField();
    private JTextField iPadYFld = new JTextField();
    private JTextField prefSizeWidthFld = new JTextField(4);
    private JTextField prefSizeHeightFld = new JTextField(4);

    private JButton gridXRelBtn = new JButton("REL");
    private JButton gridYRelBtn = new JButton("REL");
    private JButton gridWidthRelBtn = new JButton("REL");
    private JButton gridHeightRelBtn = new JButton("REL");
    private JButton gridWidthRemBtn = new JButton("REM");
    private JButton gridHeightRemBtn = new JButton("REM");
    private JToggleButton prefSizeBtn = new JToggleButton("Default");


    /**
     * Construct a new constraints editor panel.
     */
    public ConstraintsEditorPanel() {
        setLayout(new GridBagLayout());

        GridBagConstraints gbc = new GridBagConstraints();

        // First add all of the labels down the left side
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.anchor = GridBagConstraints.NORTHWEST;
        add(new JLabel("name:"), gbc);
        gbc.gridy++;
        add(new JLabel("gridx:"), gbc);
        gbc.gridy++;
        add(new JLabel("gridy:"), gbc);
        gbc.gridy++;
        add(new JLabel("gridwidth:"), gbc);
        gbc.gridy++;
        add(new JLabel("gridheight:"), gbc);
        gbc.gridy++;
        add(new JLabel("fill:"), gbc);
        gbc.gridy++;
        add(new JLabel("anchor:"), gbc);
        gbc.gridy++;
        add(new JLabel("weightx:"), gbc);
        gbc.gridy++;
        add(new JLabel("weighty:"), gbc);
        gbc.gridy++;
        add(new JLabel("ipadx:"), gbc);
        gbc.gridy++;
        add(new JLabel("ipady:"), gbc);

        // insets
        gbc.gridy += 2;
        add(new JLabel("insets:"), gbc);

        gbc.gridy += 2;
        add(new JLabel("pref size:"), gbc);

        // now add the controls in the second column
        gbc.gridx = 1;
        gbc.gridy = 0;
        gbc.gridwidth = 3;
        gbc.weightx = 1.0;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        add(nameFld, gbc);
        gbc.gridy++;
        add(gridXFld, gbc);
        gbc.gridy++;
        add(gridYFld, gbc);
        gbc.gridy++;
        add(gridWidthFld, gbc);
        gbc.gridy++;
        add(gridHeightFld, gbc);
        gbc.gridy++;
        add(fillCombo, gbc);
        gbc.gridy++;
        add(anchorCombo, gbc);
        gbc.gridy++;
        add(weightXFld, gbc);
        gbc.gridy++;
        add(weightYFld, gbc);
        gbc.gridy++;
        add(iPadXFld, gbc);
        gbc.gridy++;
        add(iPadYFld, gbc);

        // insets
        gbc.gridwidth = 1;
        gbc.gridx++;
        gbc.gridy++;
        add(insetsTopFld, gbc);
        gbc.gridx--;
        gbc.gridy++;
        add(insetsLeftFld, gbc);
        gbc.gridx += 2;
        add(insetsRightFld, gbc);
        gbc.gridx--;
        gbc.gridy++;
        add(insetsBottomFld, gbc);

        JPanel prefSizePanel = new JPanel(new GridBagLayout());
        GridBagConstraints pgbc = new GridBagConstraints();
        pgbc.gridx = 0;
        pgbc.gridy = 0;
        prefSizePanel.add(new JLabel("w"), pgbc);
        pgbc.gridx = GridBagConstraints.RELATIVE;
        prefSizePanel.add(prefSizeWidthFld, pgbc);
        prefSizePanel.add(new JLabel(" h"), pgbc);
        prefSizePanel.add(prefSizeHeightFld, pgbc);

        gbc.gridx = 1;
        gbc.gridy++;
        gbc.gridwidth = 3;
        add(prefSizePanel, gbc);

        // add shortcut buttons for RELATIVE and REMAINDER
        gbc.gridwidth = 1;
        gbc.gridx = 4;
        gbc.gridy = 1;
        gbc.weightx = 0;
        gbc.fill = GridBagConstraints.NONE;

        // REL for gridx
        gridXRelBtn.setMargin(new Insets(0,0,0,0));
        gridXRelBtn.addActionListener(this);
        add(gridXRelBtn, gbc);

        // REL for gridy
        gridYRelBtn.setMargin(new Insets(0,0,0,0));
        gridYRelBtn.addActionListener(this);
        gbc.gridy++;
        add(gridYRelBtn, gbc);

        // REL for gridwidth
        gridWidthRelBtn.setMargin(new Insets(0,0,0,0));
        gridWidthRelBtn.addActionListener(this);
        gbc.gridy++;
        add(gridWidthRelBtn, gbc);

        // REL for gridheight
        gridHeightRelBtn.setMargin(new Insets(0,0,0,0));
        gridHeightRelBtn.addActionListener(this);
        gbc.gridy++;
        add(gridHeightRelBtn, gbc);

        // REM for gridwidth
        gridWidthRemBtn.setMargin(new Insets(0,0,0,0));
        gridWidthRemBtn.addActionListener(this);
        gbc.gridx++;
        gbc.gridy--;
        add(gridWidthRemBtn, gbc);

        // REM for gridheight
        gridHeightRemBtn.setMargin(new Insets(0,0,0,0));
        gridHeightRemBtn.addActionListener(this);
        gbc.gridy++;
        add(gridHeightRemBtn, gbc);

        // Default for prefSize
        prefSizeBtn.setMargin(new Insets(0,0,0,0));
        gbc.gridwidth = 2;
        gbc.gridy += 10;
        gbc.gridx--;
        add(prefSizeBtn, gbc);

        // register event listeners
        gridXFld.addFocusListener(this); 
        gridYFld.addFocusListener(this); 
        gridWidthFld.addFocusListener(this); 
        gridHeightFld.addFocusListener(this); 
        nameFld.addFocusListener(this); 
        weightXFld.addFocusListener(this); 
        weightYFld.addFocusListener(this); 
        insetsLeftFld.addFocusListener(this); 
        insetsRightFld.addFocusListener(this);  
        insetsTopFld.addFocusListener(this);
        insetsBottomFld.addFocusListener(this);
        iPadXFld.addFocusListener(this);
        iPadYFld.addFocusListener(this);
        prefSizeWidthFld.addFocusListener(this);
        prefSizeHeightFld.addFocusListener(this);

        fillCombo.addActionListener(this);
        anchorCombo.addActionListener(this);
        prefSizeBtn.addActionListener(this);
    }


    /**
     * Set a new constraints object to edit.  If constraints is null,
     * then clear out all fields.
     * @param constraints the object to edit.
     */
    public synchronized void setConstraints(Constraints constraints) {
        if (constraints == this.constraints) {
            return;
        }

        this.constraints = constraints;
        if (constraints != null) {
            // set component values
            nameFld.setText(constraints.getName());
            gridXFld.setText(constraints.getGridX());
            gridYFld.setText(constraints.getGridY());
            gridWidthFld.setText(constraints.getGridWidth());
            gridHeightFld.setText(constraints.getGridHeight());
            fillCombo.setSelectedItem(constraints.getFill());
            anchorCombo.setSelectedItem(constraints.getAnchor());
            weightXFld.setText(constraints.getWeightX());
            weightYFld.setText(constraints.getWeightY());
            insetsLeftFld.setText(constraints.getInsetsLeft());
            insetsRightFld.setText(constraints.getInsetsRight());
            insetsTopFld.setText(constraints.getInsetsTop());
            insetsBottomFld.setText(constraints.getInsetsBottom());
            iPadXFld.setText(constraints.getIPadX());
            iPadYFld.setText(constraints.getIPadY());

            if (constraints.getUseDefaultPrefSize()) {
                prefSizeBtn.setSelected(true);
            } else {
                prefSizeBtn.setSelected(false);
            }
            prefSizeWidthFld.setText(constraints.getPrefSizeWidth());
            prefSizeHeightFld.setText(constraints.getPrefSizeHeight());
        } else {
            // clear component values
            nameFld.setText("");
            gridXFld.setText("");
            gridYFld.setText("");
            gridWidthFld.setText("");
            gridHeightFld.setText("");
            fillCombo.setSelectedIndex(0);
            anchorCombo.setSelectedIndex(0);
            weightXFld.setText("");
            weightYFld.setText("");
            insetsLeftFld.setText("");
            insetsRightFld.setText("");
            insetsTopFld.setText("");
            insetsBottomFld.setText("");
            iPadXFld.setText("");
            iPadYFld.setText("");
            prefSizeWidthFld.setText("");
            prefSizeHeightFld.setText("");
            prefSizeBtn.setSelected(true);
        }
    }


    /**
     * An unused method from the FocusListener interface.
     */
    public void focusGained(FocusEvent evt) {}


    /**
     * A method from the FocusListener interface.  Whenever focus is
     * lost on any text field, update the constraints object.
     * @param evt the Focus Event object.
     */
    public void focusLost(FocusEvent evt) {
        if (constraints == null) {
            return;
        }

        Object src = evt.getSource();
        String val = ((JTextField)src).getText();
        String oldVal = "";

        try {
            if (src == gridXFld) {
                oldVal = constraints.getGridX();
                constraints.setGridX(val);
            } else if (src == gridYFld) {
                oldVal = constraints.getGridY();
                constraints.setGridY(val);
            } else if (src == gridWidthFld) {
                oldVal = constraints.getGridWidth();
                constraints.setGridWidth(val);
            } else if (src == gridHeightFld) {
                oldVal = constraints.getGridHeight();
                constraints.setGridHeight(val);
            } else if (src == nameFld) {
                oldVal = constraints.getName();
                constraints.setName(val);
            } else if (src == weightXFld) {
                oldVal = constraints.getWeightX();
                constraints.setWeightX(val);
            } else if (src == weightYFld) {
                oldVal = constraints.getWeightY();
                constraints.setWeightY(val);
            } else if (src == insetsLeftFld) {
                oldVal = constraints.getInsetsLeft();
                constraints.setInsetsLeft(val);
            } else if (src == insetsRightFld) {
                oldVal = constraints.getInsetsRight();
                constraints.setInsetsRight(val);
            } else if (src == insetsTopFld) {
                oldVal = constraints.getInsetsTop();
                constraints.setInsetsTop(val);
            } else if (src == insetsBottomFld) {
                oldVal = constraints.getInsetsBottom();
                constraints.setInsetsBottom(val);
            } else if (src == iPadXFld) {
                oldVal = constraints.getIPadX();
                constraints.setIPadX(val);
            } else if (src == iPadYFld) {
                oldVal = constraints.getIPadY();
                constraints.setIPadY(val);
            } else if (src == prefSizeWidthFld) {
                oldVal = constraints.getPrefSizeWidth();
                constraints.setPrefSizeWidth(val);
            } else if (src == prefSizeHeightFld) {
                oldVal = constraints.getPrefSizeHeight();
                constraints.setPrefSizeHeight(val);
            }
        } catch (ConstraintsException ce) {
            // show an error dialog to the user
            GridBagExplorer.showErrorDialog(this, ce.getMessage() 
                + "\nReverting to original value.");

            // revert to the old value
            ((JTextField)src).setText(oldVal);
        }
    }

    /**
     * Implement the ActionListener interface.  This method is called
     * whenever the user hits one of the "shortcut" buttons.
     * @param evt the event which is fired whenever one of the buttons
     * is clicked.
     */
    public void actionPerformed(ActionEvent evt) {
        if (constraints == null) {
            return;
        }

        Object src = evt.getSource();
        try {
            if (src == fillCombo) {
                String fill = (String)fillCombo.getSelectedItem();
                constraints.setFill(fill);
            } else if (src == anchorCombo) {
                String anchor = (String)anchorCombo.getSelectedItem();
                constraints.setAnchor(anchor);
            } else if (src == gridXRelBtn) {
                gridXFld.setText(GBCUtil.RELATIVE_STR);
                constraints.setGridX(GBCUtil.RELATIVE_STR);
            } else if (src == gridYRelBtn) {
                gridYFld.setText(GBCUtil.RELATIVE_STR);
                constraints.setGridY(GBCUtil.RELATIVE_STR);
            } else if (src == gridWidthRelBtn) {
                gridWidthFld.setText(GBCUtil.RELATIVE_STR);
                constraints.setGridWidth(GBCUtil.RELATIVE_STR);
            } else if (src == gridHeightRelBtn) {
                gridHeightFld.setText(GBCUtil.RELATIVE_STR);
                constraints.setGridHeight(GBCUtil.RELATIVE_STR);
            } else if (src == gridWidthRemBtn) {
                gridWidthFld.setText(GBCUtil.REMAINDER_STR);
                constraints.setGridWidth(GBCUtil.REMAINDER_STR);
            } else if (src == gridHeightRemBtn) {
                gridHeightFld.setText(GBCUtil.REMAINDER_STR);
                constraints.setGridHeight(GBCUtil.REMAINDER_STR);
            } else if (src == prefSizeBtn) {
                constraints.setUseDefaultPrefSize(prefSizeBtn.isSelected());
            }
        } catch (ConstraintsException ce) {
            // show an error dialog to the user
            GridBagExplorer.showErrorDialog(this, ce.getMessage());
        }
    }
}

