package com.showmejava.gridbag;

import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.util.Hashtable;

/**
 * Utility methods for converting to/from Strings and GridBagConstraints
 * values.
 * @author Eric M. Burke
 * @version 2.0
 */
public class GBCUtil {

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String BOTH_STR = "BOTH";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String CENTER_STR = "CENTER";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String EAST_STR = "EAST";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String HORIZONTAL_STR = "HORIZONTAL";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String NONE_STR = "NONE";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String NORTH_STR = "NORTH";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String NORTHEAST_STR = "NORTHEAST";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String NORTHWEST_STR = "NORTHWEST";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String SOUTH_STR = "SOUTH";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String SOUTHEAST_STR = "SOUTHEAST";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String SOUTHWEST_STR = "SOUTHWEST";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String RELATIVE_STR = "RELATIVE";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String REMAINDER_STR = "REMAINDER";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String VERTICAL_STR = "VERTICAL";

    /**
     * A String constant which may get displayed in the GUI or in
     * the generated code.
     */
    public static final String WEST_STR = "WEST";


    /**
     * An array of all possible anchors, used by the GUI to construct
     * an option menu.
     */
    public static final String[] ALL_ANCHORS = {
        CENTER_STR, NORTH_STR, NORTHEAST_STR, EAST_STR, SOUTHEAST_STR,
        SOUTH_STR, SOUTHWEST_STR, WEST_STR, NORTHWEST_STR };

    /**
     * An array of all possible fills, used by the GUI to construct
     * an option menu.
     */
    public static final String[] ALL_FILLS = {
        NONE_STR, HORIZONTAL_STR, VERTICAL_STR, BOTH_STR };

    private static final String EMPTY_STR = "";

    private static final Hashtable stringToAnchorTbl = new Hashtable(9);
    private static final Hashtable stringToFillTbl = new Hashtable(4);

    static {
        stringToAnchorTbl.put(CENTER_STR, 
            new Integer(GridBagConstraints.CENTER));
        stringToAnchorTbl.put(EAST_STR, 
            new Integer(GridBagConstraints.EAST));
        stringToAnchorTbl.put(NORTH_STR, 
            new Integer(GridBagConstraints.NORTH));
        stringToAnchorTbl.put(NORTHEAST_STR, 
            new Integer(GridBagConstraints.NORTHEAST));
        stringToAnchorTbl.put(NORTHWEST_STR, 
            new Integer(GridBagConstraints.NORTHWEST));
        stringToAnchorTbl.put(SOUTH_STR, 
            new Integer(GridBagConstraints.SOUTH));
        stringToAnchorTbl.put(SOUTHEAST_STR, 
            new Integer(GridBagConstraints.SOUTHEAST));
        stringToAnchorTbl.put(SOUTHWEST_STR, 
            new Integer(GridBagConstraints.SOUTHWEST));
        stringToAnchorTbl.put(WEST_STR, 
            new Integer(GridBagConstraints.WEST));

        stringToFillTbl.put(NONE_STR, 
            new Integer(GridBagConstraints.NONE));
        stringToFillTbl.put(HORIZONTAL_STR, 
            new Integer(GridBagConstraints.HORIZONTAL));
        stringToFillTbl.put(VERTICAL_STR, 
            new Integer(GridBagConstraints.VERTICAL));
        stringToFillTbl.put(BOTH_STR, 
            new Integer(GridBagConstraints.BOTH));
    }


    /**
     * Convert a legal anchor constant to a String representation.
     * @param anchor one of the anchor constants from GridBagConstraints.
     * @return the constant as a String, such as "CENTER" or "NORTH".
     */
    public static String anchorToString(int anchor) 
    throws ConstraintsException {
        switch (anchor) {
        case GridBagConstraints.CENTER:
            return CENTER_STR;
        case GridBagConstraints.NORTH:
            return NORTH_STR;
        case GridBagConstraints.NORTHEAST:
            return NORTHEAST_STR;
        case GridBagConstraints.EAST:
            return EAST_STR;
        case GridBagConstraints.SOUTHEAST:
            return SOUTHEAST_STR;
        case GridBagConstraints.SOUTH:
            return SOUTH_STR;
        case GridBagConstraints.SOUTHWEST:
            return SOUTHWEST_STR;
        case GridBagConstraints.WEST:
            return WEST_STR;
        case GridBagConstraints.NORTHWEST:
            return NORTHWEST_STR;
        default:
            throw new ConstraintsException("Illegal Anchor");
        }
    }


    /**
     * Convert a legal fill constant to a String representation.
     * @param fill one of the fill constants from GridBagConstraints.
     * @return the constant as a String, such as "NONE" or "HORIZONTAL".
     */
    public static String fillToString(int fill) 
    throws ConstraintsException {
        switch (fill) {
        case GridBagConstraints.NONE:
            return NONE_STR;
        case GridBagConstraints.HORIZONTAL:
            return HORIZONTAL_STR;
        case GridBagConstraints.VERTICAL:
            return VERTICAL_STR;
        case GridBagConstraints.BOTH:
            return BOTH_STR;
        default:
            throw new ConstraintsException("Illegal fill");
        }
    }


    /**
     * Convert a legal width or height to a String representation.
     * @param gridWidthOrHeight legal values are REMAINDER, RELATIVE, or
     * an integer value.
     * @return a String value for the constant or integer.
     */
    public static String gridWidthOrHeightToString(int gridWidthOrHeight) {
        switch (gridWidthOrHeight) {
        case GridBagConstraints.REMAINDER:
            return REMAINDER_STR;
        case GridBagConstraints.RELATIVE:
            return RELATIVE_STR;
        default:
            return Integer.toString(gridWidthOrHeight);
        }
    }

    /**
     * Convert a legal gridx or gridy value to a String representation.
     * @param gridXOrY RELATIVE or an integer value.
     * @return "RELATIVE" or a String value for the int value.
     */
    public static String gridXOrYToString(int gridXOrY) {
        if (gridXOrY == GridBagConstraints.RELATIVE) {
            return RELATIVE_STR;
        }
        return Integer.toString(gridXOrY);
    }

    /**
     * Convert ipadx or ipady to a string.
     * @param iPadXOrY the value to convert.
     * @return the value formatted as a String.
     */
    public static String iPadXOrYToString(int iPadXOrY) {
        return Integer.toString(iPadXOrY);
    }


    /**
     * Convert an anchor String to one of the anchor constants defined in
     * GridBagConstraints.
     * @param anchor must be all uppercase, such as NORTH, SOUTH, etc.
     * @return one of the anchor constants defined in GridBagConstraints.
     */
    public static int stringToAnchor(String anchor) 
    throws ConstraintsException {
        Integer anchorInt = (Integer)stringToAnchorTbl.get(anchor);
        if (anchorInt != null) {
            return anchorInt.intValue();
        }
        throw new ConstraintsException("Illegal anchor");
    }


    /**
     * @param fill must be all uppercase, such as "BOTH" or "HORIZONTAL".
     * @return one of the fill constants defined in GridBagConstraints.
     */
    public static int stringToFill(String fill) 
    throws ConstraintsException {
        Integer fillInt = (Integer)stringToFillTbl.get(fill);
        if (fillInt != null) {
            return fillInt.intValue();
        }
        throw new ConstraintsException("Illegal fill");
    }


    /**
     * @param gridWidthOrHeight a String representation of gridwidth or
     * gridheight.
     * @return an int value for gridwidth or gridheight.
     */
    public static int stringToGridWidthOrHeight(String gridWidthOrHeight) 
    throws ConstraintsException {
        if (RELATIVE_STR.equals(gridWidthOrHeight)) {
            return GridBagConstraints.RELATIVE;
        } else if (REMAINDER_STR.equals(gridWidthOrHeight)) {
            return GridBagConstraints.REMAINDER;
        }
        try {
            return Integer.parseInt(gridWidthOrHeight);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal width or height");
        }
    }


    /**
     * @param gridXOrY a String value for gridx or gridy.
     * @return either RELATIVE or an int value.
     */
    public static int stringToGridXOrY(String gridXOrY) 
    throws ConstraintsException {
        try {
            return (RELATIVE_STR.equals(gridXOrY)) ? 
                    GridBagConstraints.RELATIVE : Integer.parseInt(gridXOrY);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal x or y");
        }
    }

    /**
     * @param a String value for weightx or weighty.
     * @return the String converted to a double value.
     */
    public static double stringToWeightXOrY(String weightXOrY) 
    throws ConstraintsException {
        try {
            return new Double(weightXOrY).doubleValue();
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal weight");
        }
    }


    /**
     * @param weightXOrY the weightx or weighty value.
     * @return the double converted to a String.
     */
    public static String weightXOrYToString(double weightXOrY) {
        return Double.toString(weightXOrY);
    }


    /**
     * Convert a String to ipadx or ipady.
     * @param iPadXOrY the string to convert.
     * @throws ConstraintsException if a NumberFormatException occurs.
     */
    public static int stringToIPadXOrY(String iPadXOrY)
    throws ConstraintsException {
        try {
            return Integer.parseInt(iPadXOrY);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal ipad");
        }
    }

    /**
     * @param insetsField insets.top, insets.bottom, insets.right, or
     * insets.left.
     */
    public static int stringToInsetsField(String insetsField)
    throws ConstraintsException {
        try {
            return Integer.parseInt(insetsField);
        } catch (NumberFormatException nfe) {
            throw new ConstraintsException("Illegal insets");
        }
    }

    /**
     * Convert one of the insets to a String.
     * @param insetsField the value to convert.
     * @return the insets formatted as a String.
     */
    public static String insetsFieldToString(int insetsField) {
        return Integer.toString(insetsField);
    }
}

